{% extends "admin/change_form.html" %}
{% load cms_admin cms_static %}

{% block extrahead %}
    {# in case plugins require widgets, they need to have bundle here #}
    <script src="{% static_with_version "cms/js/dist/bundle.admin.base.min.js" %}" type="text/javascript"></script>
    <script type="text/javascript">
        if (!window.jQuery) {
            window.jQuery = CMS.$;
            window.$ = window.jQuery;
        }
    </script>
    {{ block.super }}
{% endblock %}

{% block submit_buttons_bottom %}
    {% submit_row_plugin %}
{% endblock %}
