{% load i18n %}

<h3>{{ field.label }}</h3>
<ul>
    {% for choice in choices %}
        <li{% if choice.selected %} class="selected"{% endif %}>
            <a href="{{ choice.query_string|iriencode }}">{{ choice.display }}</a>
        </li>
    {% endfor %}
</ul>
