<li class="{% if active %}cms-toolbar-item-navigation-active{% endif %}{% if disabled %} cms-toolbar-item-navigation-disabled{% endif %} {{ extra_classes|join:' ' }}">
    <a{% if disabled %} tabindex="-1"{% endif %} href="{{ url }}"><span>{{ name }}<span class="cms-icon cms-icon-arrow"></span></span></a>
</li>
