/*!
 * @copyright: https://github.com/divio/django-cms
 */.cms-content-wizard .clear{clear:both;overflow:hidden}.cms-content-wizard .errornote{box-sizing:border-box;display:block;width:100%}.cms-content-wizard .choice-wrapper{clear:both;overflow:hidden;margin:-1%}.cms-content-wizard .choice{background-image:none;margin-bottom:0;border-radius:3px;color:#555;background-color:#fff;border:1px solid #ddd;background-clip:padding-box;-webkit-appearance:none;box-sizing:border-box;position:relative;text-transform:none;height:85px;padding:20px 25px!important;margin:1%!important}.cms-content-wizard .choice.focus,.cms-content-wizard .choice:focus{color:#555;background-color:#f2f2f2;border-color:#d0d0d0;text-decoration:none}.cms-content-wizard .choice:hover{color:#555;background-color:#f2f2f2;border-color:#d0d0d0;text-decoration:none}.cms-content-wizard .choice.cms-btn-active,.cms-content-wizard .choice:active{color:#555;background-color:#e6e6e6;border-color:#c3c3c3;box-shadow:inset 0 3px 5px rgba(0,0,0,.125)}.cms-content-wizard .choice.cms-btn-active.focus,.cms-content-wizard .choice.cms-btn-active:focus,.cms-content-wizard .choice.cms-btn-active:hover,.cms-content-wizard .choice:active.focus,.cms-content-wizard .choice:active:focus,.cms-content-wizard .choice:active:hover{color:#555;background-color:#d4d4d4;border-color:#9d9d9d}.cms-content-wizard .choice.cms-btn-active,.cms-content-wizard .choice:active{background-image:none}.cms-content-wizard .choice.cms-btn-disabled,.cms-content-wizard .choice.cms-btn-disabled.cms-btn-active,.cms-content-wizard .choice.cms-btn-disabled.focus,.cms-content-wizard .choice.cms-btn-disabled:active,.cms-content-wizard .choice.cms-btn-disabled:focus,.cms-content-wizard .choice.cms-btn-disabled:hover,.cms-content-wizard .choice[disabled],.cms-content-wizard .choice[disabled].cms-btn-active,.cms-content-wizard .choice[disabled].focus,.cms-content-wizard .choice[disabled]:active,.cms-content-wizard .choice[disabled]:focus,.cms-content-wizard .choice[disabled]:hover{background-color:rgba(255,255,255,.4);border-color:rgba(221,221,221,.4);color:#d5d5d5;cursor:not-allowed;box-shadow:none}.cms-content-wizard .choice.cms-btn-disabled.cms-btn-active:before,.cms-content-wizard .choice.cms-btn-disabled.focus:before,.cms-content-wizard .choice.cms-btn-disabled:active:before,.cms-content-wizard .choice.cms-btn-disabled:before,.cms-content-wizard .choice.cms-btn-disabled:focus:before,.cms-content-wizard .choice.cms-btn-disabled:hover:before,.cms-content-wizard .choice[disabled].cms-btn-active:before,.cms-content-wizard .choice[disabled].focus:before,.cms-content-wizard .choice[disabled]:active:before,.cms-content-wizard .choice[disabled]:before,.cms-content-wizard .choice[disabled]:focus:before,.cms-content-wizard .choice[disabled]:hover:before{color:rgba(85,85,85,.4)}@media (min-width:768px){.cms-content-wizard .choice{float:left!important;width:48%!important}}.cms-content-wizard .choice input{position:absolute;left:-9999px;opacity:0;visibility:hidden}.cms-content-wizard .choice strong{display:block;font-size:16px;margin-bottom:3px}.cms-content-wizard .choice .info{display:block;width:100%;white-space:nowrap;overflow:hidden;text-overflow:ellipsis}.cms-content-wizard .choice.active{border-color:#0bf;background:#e6f8ff}