# Generated by Django 1.11.13 on 2018-08-10 15:39
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0031_remove_fields'),
    ]

    operations = [
        migrations.RenameModel('Title', 'PageContent'),
        migrations.AlterModelOptions(
            name='pagecontent',
            options={'default_permissions': []},
        ),
        migrations.AlterField(
            model_name='pagecontent',
            name='page',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='pagecontent_set', to='cms.Page', verbose_name='page'),
        ),
    ]
