# Generated by Django 2.1b1 on 2018-06-20 15:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0021_auto_20180507_1432'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='page',
            options={'default_permissions': ('add', 'change', 'delete'), 'permissions': (('view_page', 'Can view page'), ('publish_page', 'Can publish page'), ('edit_static_placeholder', 'Can edit static placeholders')), 'verbose_name': 'page', 'verbose_name_plural': 'pages'},
        ),
    ]
