{% extends "admin/change_form.html" %}
{% load i18n l10n static %}

{% block title %}{% trans "Edit model" %}{% endblock %}

{% block content %}
<form action="." method="post" id="cmsplugin_form">
{% csrf_token %}
<div>
    <fieldset class="module aligned ">
        <div class="description">{{ message }}</div>
    </fieldset>
</div>
</form>
{% endblock %}
