{% load cms_tags cms_js_tags i18n sekizai_tags %}
<!DOCTYPE html>
<html class="cms-overflow">
<head>
    <meta charset="UTF-8">
    <title>{{ object.title }}</title>
    {% render_block "css" %}
</head>
<body>
    {% cms_toolbar %}
    {% render_block "js" %}
    {% language cms_toolbar.toolbar_language %}
        {% render_cms_structure_js cms_toolbar.structure_renderer object %}
    {% endlanguage %}
</body>
</html>
