/*!
 * @copyright: https://github.com/divio/django-cms
 */@font-face{font-family:django-cms-iconfont;src:url(../../fonts/4.1.0rc3/django-cms-iconfont.eot);src:url(../../fonts/4.1.0rc3/django-cms-iconfont.eot#iefix) format("eot"),url(../../fonts/4.1.0rc3/django-cms-iconfont.woff2) format("woff2"),url(../../fonts/4.1.0rc3/django-cms-iconfont.woff) format("woff"),url(../../fonts/4.1.0rc3/django-cms-iconfont.ttf) format("truetype"),url(../../fonts/4.1.0rc3/django-cms-iconfont.svg#django-cms-iconfont) format("svg");font-weight:400;font-style:normal}.cms-icon{display:inline-block;font:normal normal normal 16px/1 django-cms-iconfont;text-rendering:auto;transform:translate(0,0);-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.cms-icon-advanced-settings:before{content:"\E001"}.cms-icon-alias:before{content:"\E002"}.cms-icon-apphook:before{content:"\E003"}.cms-icon-archive:before{content:"\E004"}.cms-icon-arrow-right:before{content:"\E005"}.cms-icon-arrow-wide:before{content:"\E006"}.cms-icon-arrow:before{content:"\E007"}.cms-icon-bin:before{content:"\E008"}.cms-icon-broadcast-off:before{content:"\E009"}.cms-icon-broadcast-on:before{content:"\E00A"}.cms-icon-check-circle:before{content:"\E00B"}.cms-icon-check-o:before{content:"\E00C"}.cms-icon-check-square:before{content:"\E00D"}.cms-icon-check:before{content:"\E00E"}.cms-icon-close:before{content:"\E00F"}.cms-icon-cogs:before{content:"\E010"}.cms-icon-comment:before{content:"\E011"}.cms-icon-compare:before{content:"\E012"}.cms-icon-copy:before{content:"\E013"}.cms-icon-cut:before{content:"\E014"}.cms-icon-edit-new:before{content:"\E015"}.cms-icon-edit:before{content:"\E016"}.cms-icon-eye:before{content:"\E017"}.cms-icon-forbidden:before{content:"\E018"}.cms-icon-handler:before{content:"\E019"}.cms-icon-highlight:before{content:"\E01A"}.cms-icon-home:before{content:"\E01B"}.cms-icon-info:before{content:"\E01C"}.cms-icon-layers:before{content:"\E01D"}.cms-icon-list-ol:before{content:"\E01E"}.cms-icon-loader:before{content:"\E01F"}.cms-icon-lock:before{content:"\E020"}.cms-icon-logo:before{content:"\E021"}.cms-icon-manage-versions:before{content:"\E022"}.cms-icon-menu:before{content:"\E023"}.cms-icon-minimize:before{content:"\E024"}.cms-icon-minus-circle:before{content:"\E025"}.cms-icon-minus-square-o:before{content:"\E026"}.cms-icon-minus-square:before{content:"\E027"}.cms-icon-minus:before{content:"\E028"}.cms-icon-moderate:before{content:"\E029"}.cms-icon-moon:before{content:"\E02A"}.cms-icon-paste:before{content:"\E02B"}.cms-icon-pencil:before{content:"\E02C"}.cms-icon-pin:before{content:"\E02D"}.cms-icon-plugins:before{content:"\E02E"}.cms-icon-plus-circle:before{content:"\E02F"}.cms-icon-plus-square-o:before{content:"\E030"}.cms-icon-plus:before{content:"\E031"}.cms-icon-publish:before{content:"\E032"}.cms-icon-puzzle:before{content:"\E033"}.cms-icon-redo:before{content:"\E034"}.cms-icon-rename:before{content:"\E035"}.cms-icon-scissors:before{content:"\E036"}.cms-icon-search:before{content:"\E037"}.cms-icon-settings:before{content:"\E038"}.cms-icon-sitemap:before{content:"\E039"}.cms-icon-squares:before{content:"\E03A"}.cms-icon-sun:before{content:"\E03B"}.cms-icon-undo:before{content:"\E03C"}.cms-icon-unlock:before{content:"\E03D"}.cms-icon-unpublish:before{content:"\E03E"}.cms-icon-view:before{content:"\E03F"}.cms-icon-window:before{content:"\E040"}.cms-welcome-bg{background:#f2f2f2}.cms-welcome{color:#454545;width:60%;min-width:320px;margin:0 auto}.cms-welcome a{color:#0bf}.cms-welcome a:active,.cms-welcome a:focus,.cms-welcome a:hover{text-decoration:underline}.cms-welcome .cms-hidden{display:none}.cms-welcome .cms-welcome-footer,.cms-welcome .cms-welcome-section{box-sizing:border-box;width:60%;min-width:320px;margin:0 auto}.cms-welcome .cms-welcome-header{text-align:center}.cms-welcome .cms-welcome-heading{color:#999;font-size:22px;font-weight:200;text-align:center;padding:40px 0 15px;border-bottom:1px solid #ddd}.cms-welcome .cms-welcome-heading .cms-icon{color:#693}.cms-welcome .cms-welcome-logo,.cms-welcome .cms-welcome-logo:focus,.cms-welcome .cms-welcome-logo:hover{display:inline-block;background-image:url(../../fonts/src/logo.svg);background-repeat:no-repeat;background-position:center center;height:35px;width:200px;font-size:35px;text-decoration:none;margin:50px auto 30px}.cms-welcome .cms-welcome-section{box-shadow:0 0 5px #ddd;padding:20px;margin-bottom:5px;background:#fff}.cms-welcome .cms-welcome-section h2{font-size:16px;font-weight:700;margin:0 0 15px}.cms-welcome .cms-welcome-section p{padding:5px 0}.cms-welcome .cms-welcome-links{padding:10px 5px;margin:0 0 15px;border-bottom:1px solid #ddd}.cms-welcome .cms-welcome-links a{padding:0 5px}.cms-welcome .cms-welcome-notes{font-size:12px;line-height:16px;color:#999;padding:5px 10px}