// #############################################################################
// CUSTOM

// https://github.com/Compass/compass/blob/stable/core/stylesheets/compass/typography/text/_replacement.scss
@mixin hide-content() {
    $approximate-em-value: 12px / 1em;
    $wider-than-any-screen: -9999em;
    text-indent: $wider-than-any-screen * $approximate-em-value;
    overflow: hidden;
    text-align: left;
}
