<a href="{{ url }}" class="cms-btn{% if active %} cms-btn-active{% endif %}{% if disabled %} cms-btn-disabled{% endif %} {{ extra_classes|join:' ' }}" data-rel="modal"{% if on_close %} data-on-close="{{ on_close }}"{% endif %}>{{ name }}</a>
