{% extends 'admin/cms/mail/base.html' %}
{% load i18n %}

{% block content %}
    <table>
        <tr>
            <td>{% trans 'Username:' %}</td>
            <td>{{ user.username }}</td>
        </tr>
        <tr>
            <td>{% trans 'Password:' %}</td>
            <td>{{ password }}</td>
        </tr>
    </table>
{% endblock %}