# Generated by Django 1.11 on 2018-08-30 15:49
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0033_placeholder_source_data_migration'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='placeholder',
            options={'default_permissions': [], 'permissions': (('use_structure', 'Can use Structure mode'),)},
        ),
        migrations.RemoveField(
            model_name='pagecontent',
            name='placeholders',
        ),
    ]
