# Generated by Django 1.11 on 2018-07-02 16:17
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0025_remove_plugin_tree_fields'),
    ]

    operations = [
        migrations.AddField(
            model_name='title',
            name='placeholders',
            field=models.ManyToManyField(editable=False, to='cms.Placeholder'),
        ),
        # Temporary field used by migration 0027
        migrations.AddField(
            model_name='placeholder',
            name='title_id',
            field=models.PositiveSmallIntegerField(null=True),
        ),
    ]
