{% load i18n sekizai_tags cms_tags %}

{% url 'logout' as logout_url %}
<form action="{% url 'cms_login' %}?next={{ request.path }}"
    class="cms-toolbar-item cms-form-login" method="post">
    {% csrf_token %}
    <label{% if request.GET.cms_toolbar_login_error %} class="cms-error"{% endif %}>
        <span>{% trans login_form.username.label %}</span>
        {{ login_form.username }}
    </label>
    <label{% if request.GET.cms_toolbar_login_error %} class="cms-error"{% endif %}>
        <span>{% trans login_form.password.label %}</span>
        {{ login_form.password }}
    </label>
    <label>
        <input class="cms-btn cms-btn-action" type="submit" value="{% trans 'Login' %}">
    </label>
</form>
