<a href="javascript: void 0" class="cms-btn cms-dropdown-toggle {% if active %} cms-btn-active{% endif %}{% if disabled %} cms-btn-disabled{% endif %} {{ extra_classes|join:' ' }}">
    {{ name }}
    <span class="cms-dropdown-caret"></span>
</a>

