# Generated by Django 1.11.12 on 2018-05-29 14:39
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0024_set_plugin_absolute_position'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='cmsplugin',
            options={'ordering': ('position',)},
        ),
        migrations.AlterField(
            model_name='cmsplugin',
            name='position',
            field=models.SmallIntegerField(default=1, editable=False, verbose_name='position'),
        ),
        migrations.AlterUniqueTogether(
            name='cmsplugin',
            unique_together=set([('placeholder', 'language', 'position')]),
        ),
        migrations.AddIndex(
            model_name='cmsplugin',
            index=models.Index(fields=['placeholder', 'language', 'position'], name='cms_cmsplug_placeho_3f2b0f_idx'),
        ),
        migrations.RemoveField(
            model_name='cmsplugin',
            name='depth',
        ),
        migrations.RemoveField(
            model_name='cmsplugin',
            name='numchild',
        ),
        migrations.RemoveField(
            model_name='cmsplugin',
            name='path',
        ),
    ]
