# Generated by Django 1.11 on 2018-07-03 09:39
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0027_title_placeholders_data_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='page',
            name='placeholders',
        ),
        migrations.RemoveField(
            model_name='placeholder',
            name='title_id',
        ),
    ]
