{% extends "cms/wizards/base.html" %}
{% load i18n admin_urls %}

{% block wizard %}
    <h1>{% trans "Create" %} {{ wizard_entry.title }}</h1>

    <form action="{% url 'cms_wizard_create' %}" method="post" enctype="multipart/form-data" novalidate>
        {% csrf_token %}

        {{ wizard.management_form }}

        {% if form.errors %}
            <p class="errornote">
                {% blocktrans count form.errors|length as counter %}Please correct the error below.{% plural %}Please correct the errors below.{% endblocktrans %}
            </p>
        {% endif %}

        <div>
            {% include "cms/wizards/includes/form_fields.html" %}
        </div>
        <div class="submit-row">
            <input type="submit" class="default cms-btn cms-btn-action" value="{% trans "Create" %}">
            <button type="submit" name="wizard_goto_step"
                class="cms-btn cms-btn-secondary cms-btn-group" value="{{ wizard.steps.prev }}">
                {% trans "Back" %}
            </button>
        </div>
    </form>
{% endblock %}
