{% spaceless %}
<a class="btn{% if disabled %} inactive{% endif %} cms-form-{{ method }}-method{% if burger_menu %} cms-burger-action-btn{% endif %} cms-action-btn{% block name %}{% if name %} cms-action-{{ name }}{% endif %}{% endblock %}{% if action|default_if_none:True %} js-action{% endif %} {% if keepsideframe|default_if_none:True %}js-keep-sideframe{% else %}js-close-sideframe{% endif %}"{% if not disabled %} href="{{ url }}"{% endif %} title="{% block title %}{{ title }}{% endblock %}">
    <span class="cms-icon cms-icon-{% block icon %}{{ icon }}{% endblock %}"></span>
</a>
{% endspaceless %}
